@@ -21,12 +21,12 @@ import shutil
 import tempfile
 from urllib import parse as urlparse
 
-from ironic_lib import utils as ironic_utils
 import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
+from oslo_utils import netutils
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
@@ -108,7 +108,7 @@ def _link_mac_pxe_configs(task, ipxe_enabled=False):
     """
 
     def create_link(mac_path):
-        ironic_utils.unlink_without_raise(mac_path)
+        utils.unlink_without_raise(mac_path)
         relative_source_path = os.path.relpath(
             pxe_config_file_path, os.path.dirname(mac_path))
         utils.create_link_without_raise(relative_source_path, mac_path)
@@ -159,7 +159,7 @@ def _link_ip_address_pxe_configs(task, ipxe_enabled=False):
         ip_addrs = []
     for port_ip_address in ip_addrs:
         ip_address_path = _get_pxe_ip_address_path(port_ip_address)
-        ironic_utils.unlink_without_raise(ip_address_path)
+        utils.unlink_without_raise(ip_address_path)
         relative_source_path = os.path.relpath(
             pxe_config_file_path, os.path.dirname(ip_address_path))
         utils.create_link_without_raise(relative_source_path,
@@ -351,11 +351,9 @@ def create_pxe_config(task, pxe_options, template=None, ipxe_enabled=False):
         pxe_config_root_tag = '(( ROOT ))'
         pxe_config_disk_ident = '(( DISK_IDENTIFIER ))'
 
-        # Determine the appropriate commands based on the CPU architecture
-        arch = task.node.properties.get('cpu_arch', 'x86_64')
         commands = {
-            'linux_cmd': 'linuxefi' if arch != 'aarch64' else 'linux',
-            'initrd_cmd': 'initrdefi' if arch != 'aarch64' else 'initrd'
+            'linux_cmd': 'linux',
+            'initrd_cmd': 'initrd'
         }
         pxe_options.update(commands)
     else:
@@ -431,18 +429,19 @@ def clean_up_pxe_config(task, ipxe_enabled=False):
             except exception.FailedToGetIPAddressOnPort:
                 continue
             # Cleaning up config files created for grub2.
-            ironic_utils.unlink_without_raise(ip_address_path)
+            utils.unlink_without_raise(ip_address_path)
 
     for port in task.ports:
         client_id = port.extra.get('client-id')
         # syslinux, ipxe, etc.
-        ironic_utils.unlink_without_raise(
-            _get_pxe_mac_path(port.address, client_id=client_id,
+        utils.unlink_without_raise(
+            _get_pxe_mac_path(port.address,
+                              client_id=client_id,
                               ipxe_enabled=ipxe_enabled))
         # Grub2 MAC address based configuration
         for path in _get_pxe_grub_mac_path(port.address,
                                            ipxe_enabled=ipxe_enabled):
-            ironic_utils.unlink_without_raise(path)
+            utils.unlink_without_raise(path)
     utils.rmtree_without_raise(os.path.join(_get_root_dir(ipxe_enabled),
                                             task.node.uuid))
 
@@ -1090,10 +1089,8 @@ def get_volume_pxe_options(task):
         return __return_item_or_first_if_list(properties.get(key + 's', ''))
 
     def __format_portal(portal, iqn, lun):
-        if ':' in portal:
-            host, port = portal.split(':')
-        else:
-            host = portal
+        host, port = netutils.parse_host_port(portal)
+        if port is None:
             port = ''
         return ("iscsi:%(host)s::%(port)s:%(lun)x:%(iqn)s" %
                 {'host': host, 'port': port, 'lun': lun, 'iqn': iqn})
@@ -1299,7 +1296,10 @@ class TFTPImageCache(image_cache.ImageCache):
             # MiB -> B
             cache_size=CONF.pxe.image_cache_size * 1024 * 1024,
             # min -> sec
-            cache_ttl=CONF.pxe.image_cache_ttl * 60)
+            cache_ttl=CONF.pxe.image_cache_ttl * 60,
+            # This is a cache for kernel, initramfs and anaconda artifacts,
+            # none of which can be validated
+            disable_validation=True)
 
 
 def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
@@ -1329,8 +1329,16 @@ def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
     LOG.debug("Fetching necessary kernel and ramdisk for node %s",
               node.uuid)
     images_info = list(t_pxe_info.values())
+
+    # Call the central auth override lookup, but signal it is not
+    # for *user* direct artifacts, i.e. this is system related
+    # activity as we're getting TFTP Image cache artifacts.
+    img_service_auth = service.get_image_service_auth_override(
+        task.node, permit_user_auth=False)
+
     deploy_utils.fetch_images(ctx, TFTPImageCache(), images_info,
-                              CONF.force_raw_images)
+                              CONF.force_raw_images,
+                              image_auth_data=img_service_auth)
     if CONF.pxe.file_permission:
         for info in images_info:
             os.chmod(info[1], CONF.pxe.file_permission)
@@ -1349,7 +1357,7 @@ def clean_up_pxe_env(task, images_info, ipxe_enabled=False):
     """
     for label in images_info:
         path = images_info[label][1]
-        ironic_utils.unlink_without_raise(path)
+        utils.unlink_without_raise(path)
 
     clean_up_pxe_config(task, ipxe_enabled=ipxe_enabled)
     TFTPImageCache().clean_up()
